-- Jonathan Frech, 2019-11-29, 2019-11-30, 2019-12-01, 2019-12-02, 2019-12-03,
--                 2019-12-04, 2019-12-08, 2019-12-22, 2019-12-23, 2019-12-24,
--                 2019-12-25, 2019-12-26, 2021-09-03
-- Brainfuck Compiler Project (BCP)

{--- Unfinished Future Plans ---------------------------------------------------
    * file inclusion mechanism
    * streamline type/hints system, esp. boolean hints
    * command-line interface
    * testing and refactoring -}

{--- Compilation Pipeline ------------------------------------------------------
    BMA: Brainfuck Macro Assembler
    MAL: Macro Assembly Language
     v   via evaluation (akin to assembly)
    TAL: Tape Abstraction Language
    BTA: Brainfuck Tape Abstraction
     v   via concretion
    BF.: BrainFuck language
    BFL: BrainFuck Language
     v   via transpilation
    C
     v   via an external C compiler~> Machine Code
    Machine Code -}


{-# LANGUAGE MultiWayIf #-}

import Data.Maybe (mapMaybe)
import System.Environment (getArgs)
import System.Exit (die, exitSuccess)

import Brainfuck
import Error
import MacroAssembler
import PArg
import SourceMarker
import TapeAbstraction
import Util (safeWriteFile, getDate)


main :: IO ()
main = do args <- getArgs
          let pargs = parseArgs args
          let pargSrcs = mapMaybe fromPArgSrc pargs
          let pargErrors = mapMaybe fromPArgError pargs

          if | length pargErrors /= 0 -> printErrors pargErrors
             | PArgHelp `elem` pargs  -> printHelp
             | otherwise -> case pargSrcs of
                [src] -> do eexprs <- include src
                            let out = src ++ ".transpiled.c"
                            case eexprs of
                                Left err    -> printError err
                                Right exprs -> case evaluate $ Expr (FileMarker src) exprs of
                                    Left errs        -> printErrors errs
                                    Right tapeinstrs -> case concrete tapeinstrs of
                                        Left errs -> printErrors errs
                                        Right bf' -> do let bf = optimizeBrainfuck bf'
                                                        let bfSrc = brainfuckSource bf
                                                        putStrLn $ bfSrc
                                                        putStrLn $ "(" ++ show (length (bfSrc)) ++ " characters)"
                                                        putStrLn $ "Naive brainfuck optimization strength: " ++ show (length bf' - length bf)
                                                        putStrLn ""

                                                        date <- getDate
                                                        success <- safeWriteFile out $ transpile (date,src) bf
                                                        case success of
                                                            False -> printError . Error (Marker "Main IO") $ "Failed two write file " ++ show out ++ "."
                                                            True  -> putStrLn $ "Successfully wrote file " ++ show out ++ "."
                _     -> printError . Error (Marker "PArg") $ "Please specify exactly one source file."

    where printErrors :: [Error] -> IO ()
          printErrors errs = die $ unlines ("=== ERROR ===" : (prettyError <$> errs))
          printError :: Error -> IO ()
          printError = printErrors . pure

          printHelp :: IO ()
          printHelp = do putStrLn "No help."
                         exitSuccess
