{-# LANGUAGE NamedFieldPuns #-}

module Error where

import SourceMarker


data Error = Error SourceMarker String
           deriving (Show, Eq)


ansiBld = "\x1b[1m"
ansiErr = "\x1b[1m\x1b[38;5;9m"
ansiClr = "\x1b[0m"

prettyError :: Error -> String
prettyError (Error (SourceMarker { filepath, character, line, column, source }) err)
    = case drop line (lines source) of
        [] -> "Invalid marker."
        (l:_) -> ansiBld ++ "Error in '" ++ filepath ++ "', character " ++ show (character+1) ++ ", line " ++ show (line+1) ++ ", column " ++ show (column+1) ++ ansiClr ++ "\n"
              ++ lineNumberFiller ++ "\n"
              ++ lineNumber ++ l ++ "\n"
              ++ lineNumberFiller ++ lineErrorMarker ++ "\n"
              ++ lineNumberSpace ++ lineErrorMarker ++ (ansi ansiError $ " Error: " ++ err) ++ "\n"
    where lineNumber = ansi ansiLineNumber $ show (line+1) ++ " | "
          lineNumberFiller = ansi ansiLineNumber $ replicate (length (show (line+1))) ' ' ++ " | "
          lineNumberSpace = replicate (length (show (line+1)) + 3) ' '
          lineErrorMarker = replicate column ' ' ++ ansi ansiError "^"

          ansiReset = "\x1b[0m"
          ansiLineNumber = "\x1b[38;5;2m"
          ansiError = "\x1b[1m\x1b[38;5;9m"
          ansi col msg = col ++ msg ++ ansiReset

prettyError (Error Abyss err) = "Abyss! error: " ++ err
prettyError (Error (Marker mrk) err) = ansiBld ++ "Error in " ++ show mrk ++ ":" ++ ansiClr ++ " " ++ ansiErr ++ err ++ ansiClr
prettyError (Error (FileMarker fp) err) = bld ++ "Error in file " ++ "" ++ show fp ++ bld ++ ": " ++ clr ++ er ++ err ++ clr
    where clr = "\x1b[0m"
          er  = "\x1b[1m\x1b[38;5;9m"
          bld = "\x1b[1m"
