module PArg where

import Error
import SourceMarker


data PArg = PArgHelp
          | PArgSrc FilePath
          | PArgError Error
          deriving Eq

fromPArgError :: PArg -> Maybe Error
fromPArgError (PArgError err) = Just err
fromPArgError _               = Nothing

fromPArgSrc :: PArg -> Maybe FilePath
fromPArgSrc (PArgSrc fp) = Just fp
fromPArgSrc _            = Nothing


parseArgs' :: Bool -> [String] -> [PArg]
parseArgs' _ [] = []
parseArgs' True ("--":args) = parseArgs' False args
parseArgs' True (arg:args) = case arg of
        "-h"         -> go PArgHelp
        "--help"     -> go PArgHelp
        ""           -> parseArgs' True args
        ('-':'-':as) -> go . PArgError . Error (Marker "PArg.parseArgs'") $ "Unknown argument " ++ show arg ++ "."
        ('-':a:[])   -> go . PArgError . Error (Marker "PArg.parseArgs'") $ "Unknown argument " ++ show arg ++ "."
        ('-':a:as)   -> parseArgs' True $ [('-':a:[]), ('-':as)] ++ args
        src          -> go $ PArgSrc src
    where go :: PArg -> [PArg]
          go parg = parg : parseArgs' True args

parseArgs' False (src:args) = PArgSrc src : parseArgs' False args

parseArgs :: [String] -> [PArg]
parseArgs = parseArgs' True
