{-# LANGUAGE NamedFieldPuns #-}

module SourceMarker where


data SourceMarker
    = SourceMarker { filepath :: FilePath
                   , character :: Int
                   , line :: Int
                   , column :: Int
                   , source :: String }
    | Abyss
    | FileMarker FilePath
    | Marker String
    deriving Eq

instance Show SourceMarker where
    show (SourceMarker { line, column }) = "L" ++ show line ++ "C" ++ show column
    show Abyss = "Abyss"
    show (Marker emsg) = ">>> " ++ emsg
    show (FileMarker fp) = fp


advanceMarker :: Char -> SourceMarker -> SourceMarker
advanceMarker c sm@(SourceMarker { character, line, column })
    | c == '\n' = sm { character = character + 1, line = line + 1, column = 0 }
    | otherwise = sm { character = character + 1,                  column = column + 1 }
advanceMarker _ Abyss = Abyss
advanceMarker _ m = m

advanceMarker' :: String -> SourceMarker -> SourceMarker
advanceMarker' "" sm     = sm
advanceMarker' (c:cs) sm = advanceMarker' cs $ advanceMarker c sm
