module Util where

import Control.Exception (try)
import Data.List (genericLength)
import Data.Time (getCurrentTime)


getDate :: IO String
getDate = do now <- getCurrentTime
             return $ show now

safeTail :: [a] -> [a]
safeTail []     = []
safeTail (_:xs) = xs

safeReadFile :: FilePath -> IO (Maybe String)
safeReadFile fp
    = do let attempt = try :: IO String -> IO (Either IOError String)
         esrc <- attempt $ readFile fp
         case esrc of
             Left  _   -> return Nothing
             Right src -> return $ Just src

safeWriteFile :: FilePath -> String -> IO Bool
safeWriteFile fp txt
    = do let attempt = try :: IO () -> IO (Either IOError ())
         enull <- attempt $ writeFile fp txt
         case enull of
             Left  _ -> return False
             Right _ -> return True

unwordsBreaking :: Integer -> [String] -> String
unwordsBreaking width = go 0
    where go :: Integer -> [String] -> String
          go _       []     = ""
          go current (w:ws) | current == 0 = w ++ go (genericLength w) ws
                            | 1 + current + genericLength w <= width = " " ++ w ++ go (1 + current + genericLength w) ws
                            | otherwise = "\n" ++ go 0 (w:ws)

indent :: Int -> String -> String
indent n = unlines . map (replicate n ' ' ++) . lines

fixpoint :: Eq a => (a -> a) -> a -> a
fixpoint f x | f x == x  = x
             | otherwise = fixpoint f $ f x

mapFst f (x, y) = (f x, y)
